'''
네, 그래프 설정(축 범위, 스케일 등)의 커스터마이징에 대해 설명드리겠습니다. 이전 코드를 기반으로 그래프 커스터마이징 기능을 추가한 버전입니다.
'''

import sys
import socket
import json
import numpy as np
from collections import deque

# from PyQt5.QtWidgets import (QApplication, QMainWindow, QVBoxLayout, QWidget, 
#                            QHBoxLayout, QLabel, QSpinBox, QPushButton, 
#                            QDoubleSpinBox, QComboBox, QCheckBox, QGroupBox)
# from PyQt5.QtCore import QTimer

from PyQt6.QtWidgets import (QApplication, QMainWindow, QVBoxLayout, QWidget, 
                           QHBoxLayout, QLabel, QSpinBox, QPushButton, 
                           QDoubleSpinBox, QComboBox, QCheckBox, QGroupBox)
from PyQt6.QtCore import QTimer

from pyqtgraph import PlotWidget

class CustomizableFFTVisualizer(QMainWindow):
    def __init__(self, port=5000, buffer_size=10):
        super().__init__()
        self.setWindowTitle('Customizable FFT Spectrum Analyzer')
        self.setGeometry(100, 100, 1200, 800)
        
        # 기본 설정값
        self.buffer_size = buffer_size
        self.data_buffer = deque(maxlen=buffer_size)
        self.frequencies = None
        self.auto_scale = True
        self.log_scale = False
        
        self.setup_ui()
        self.setup_network(port)
        self.setup_timers()

    def setup_ui(self):
        main_widget = QWidget()
        self.setCentralWidget(main_widget)
        layout = QVBoxLayout(main_widget)
        
        # 컨트롤 패널 그룹화
        control_panel = QWidget()
        control_layout = QHBoxLayout(control_panel)
        layout.addWidget(control_panel)
        
        # X축 설정 그룹
        x_group = QGroupBox("X-Axis (Frequency)")
        x_layout = QVBoxLayout(x_group)
        
        # X축 범위 설정
        x_range_layout = QHBoxLayout()
        x_range_layout.addWidget(QLabel("Range:"))
        self.x_min = QDoubleSpinBox()
        self.x_max = QDoubleSpinBox()
        self.x_min.setRange(0, 100000)
        self.x_max.setRange(0, 100000)
        self.x_min.setValue(0)
        self.x_max.setValue(1000)
        x_range_layout.addWidget(self.x_min)
        x_range_layout.addWidget(self.x_max)
        x_layout.addLayout(x_range_layout)
        
        # X축 스케일 설정
        self.x_log = QCheckBox("Log Scale")
        x_layout.addWidget(self.x_log)
        
        control_layout.addWidget(x_group)
        
        # Y축 설정 그룹
        y_group = QGroupBox("Y-Axis (Amplitude)")
        y_layout = QVBoxLayout(y_group)
        
        # Y축 범위 설정
        y_range_layout = QHBoxLayout()
        y_range_layout.addWidget(QLabel("Range:"))
        self.y_min = QDoubleSpinBox()
        self.y_max = QDoubleSpinBox()
        self.y_min.setRange(-1000, 1000)
        self.y_max.setRange(-1000, 1000)
        self.y_min.setValue(0)
        self.y_max.setValue(100)
        y_range_layout.addWidget(self.y_min)
        y_range_layout.addWidget(self.y_max)
        y_layout.addLayout(y_range_layout)
        
        # Y축 스케일 설정
        self.y_log = QCheckBox("Log Scale")
        y_layout.addWidget(self.y_log)
        
        control_layout.addWidget(y_group)
        
        # 디스플레이 설정 그룹
        display_group = QGroupBox("Display Settings")
        display_layout = QVBoxLayout(display_group)
        
        # 자동 스케일링
        self.auto_scale_cb = QCheckBox("Auto Scale")
        self.auto_scale_cb.setChecked(True)
        display_layout.addWidget(self.auto_scale_cb)
        
        # 그리드 설정
        self.grid_cb = QCheckBox("Show Grid")
        self.grid_cb.setChecked(True)
        display_layout.addWidget(self.grid_cb)
        
        # 색상 설정
        color_layout = QHBoxLayout()
        color_layout.addWidget(QLabel("Line Color:"))
        self.color_combo = QComboBox()
        self.color_combo.addItems(['Blue', 'Red', 'Green', 'Yellow', 'White', 'Cyan'])
        color_layout.addWidget(self.color_combo)
        display_layout.addLayout(color_layout)
        
        # 선 두께 설정
        width_layout = QHBoxLayout()
        width_layout.addWidget(QLabel("Line Width:"))
        self.width_spin = QSpinBox()
        self.width_spin.setRange(1, 10)
        self.width_spin.setValue(1)
        width_layout.addWidget(self.width_spin)
        display_layout.addLayout(width_layout)
        
        control_layout.addWidget(display_group)
        
        # 적용 버튼
        apply_button = QPushButton("Apply Settings")
        apply_button.clicked.connect(self.apply_settings)
        control_layout.addWidget(apply_button)
        
        # 플롯 위젯 설정
        self.plot_widget = PlotWidget()
        layout.addWidget(self.plot_widget)
        
        self.setup_plot()
        
        # 이벤트 연결
        self.auto_scale_cb.stateChanged.connect(self.toggle_auto_scale)
        self.grid_cb.stateChanged.connect(self.toggle_grid)
        self.color_combo.currentTextChanged.connect(self.update_line_color)
        self.width_spin.valueChanged.connect(self.update_line_width)
        self.x_log.stateChanged.connect(lambda: self.apply_settings())
        self.y_log.stateChanged.connect(lambda: self.apply_settings())

    def setup_plot(self):
        self.plot_widget.setBackground('black')
        self.plot_widget.showGrid(x=True, y=True)
        self.plot_widget.setLabel('left', 'Amplitude')
        self.plot_widget.setLabel('bottom', 'Frequency (Hz)')
        self.plot_widget.setTitle('FFT Spectrum', color='white', size='12pt')
        
        # 데이터 곡선
        self.curve = self.plot_widget.plot(pen='b')

    def setup_network(self, port):
        self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server_socket.bind(('localhost', port))
        self.server_socket.listen(1)
        self.server_socket.setblocking(False)
        
        self.client_socket = None
        self.connected = False

    def setup_timers(self):
        # 연결 타이머
        self.connect_timer = QTimer()
        self.connect_timer.timeout.connect(self.try_connect)
        self.connect_timer.start(100)
        
        # 업데이트 타이머
        self.update_timer = QTimer()
        self.update_timer.timeout.connect(self.update_plot)
        self.update_timer.start(50)

    def try_connect(self):
        if not self.connected:
            try:
                self.client_socket, _ = self.server_socket.accept()
                self.client_socket.setblocking(False)
                self.connected = True
                self.statusBar().showMessage('Connected!')
                self.connect_timer.stop()
            except BlockingIOError:
                pass
            except Exception as e:
                self.statusBar().showMessage(f'Connection error: {e}')

    def apply_settings(self):
        # 축 범위 설정
        if not self.auto_scale_cb.isChecked():
            self.plot_widget.setXRange(self.x_min.value(), self.x_max.value())
            self.plot_widget.setYRange(self.y_min.value(), self.y_max.value())
        
        # 로그 스케일 설정
        self.plot_widget.setLogMode(x=self.x_log.isChecked(), 
                                  y=self.y_log.isChecked())
        
        # 그리드 설정
        self.plot_widget.showGrid(x=self.grid_cb.isChecked(), 
                                y=self.grid_cb.isChecked())
        
        # 선 스타일 업데이트
        self.update_line_style()

    def toggle_auto_scale(self, state):
        self.auto_scale = state
        if state:
            self.plot_widget.enableAutoRange()
        else:
            self.apply_settings()

    def toggle_grid(self, state):
        self.plot_widget.showGrid(x=state, y=state)

    def update_line_color(self, color):
        self.update_line_style()

    def update_line_width(self):
        self.update_line_style()

    def update_line_style(self):
        color = self.color_combo.currentText().lower()
        width = self.width_spin.value()
        self.curve.setPen(color, width=width)

    def update_plot(self):
        if not self.connected:
            return

        try:
            # data = self.client_socket.recv(4096).decode()

            k_buff_size = 4096
            data = b''
            while True:
                part = self.client_socket.recv(k_buff_size)
                data += part
                if len(part) < k_buff_size:
                    break
            data = data.decode()

            if data:
                fft_data = json.loads(data)
                frequencies = np.array(fft_data['frequencies'])
                amplitudes = np.array(fft_data['amplitudes'])
                
                if self.frequencies is None:
                    self.frequencies = frequencies
                
                self.data_buffer.append(amplitudes)
                self.curve.setData(frequencies, amplitudes)
                
        except BlockingIOError:
            pass
        except Exception as e:
            self.statusBar().showMessage(f'Error during update: {e}')
            self.connected = False
            self.connect_timer.start()

    # 네트워크 관련 메서드들은 이전 버전과 동일...

def main():
    app = QApplication(sys.argv)
    app.setStyle('Fusion')
    window = CustomizableFFTVisualizer()
    window.show()

    # PyQt5
    # sys.exit(app.exec_())

    # PyQt6
    sys.exit(app.exec())


if __name__ == '__main__':
    main()


'''
Customizable FFT Spectrum VisualizerClick to open code
이 버전의 주요 기능과 특징을 설명드리겠습니다:

X축(주파수) 커스터마이징:

범위 수동 설정 (최소/최대 값)
로그 스케일 토글
자동 스케일링 옵션


Y축(진폭) 커스터마이징:

범위 수동 설정
로그 스케일 토글
자동 스케일링 옵션


디스플레이 설정:

그리드 표시/숨기기
선 색상 선택 (여러 색상 옵션)
선 두께 조절
배경색 설정


사용성 개선:

설정들을 논리적 그룹으로 구성
실시간 미리보기
모든 설정을 한 번에 적용하는 버튼
직관적인 컨트롤 배치



마지막으로 데이터 저장 기능에 대해 설명드릴까요?


HS: 이 코드와 바로 직전 코드는 PyQt5로 GUI를 만드는 거죠? GUI는 FFT의 결과를 표시하는 거고요. 이번 코드는 직전 코드에 비해서 위에 언급한 기능과 특징이 추가된 것이고요. 제대로 이해한 건가요? 
저장 기능에 대해 설명을 부탁합니다.

CA: 네, 정확히 이해하셨습니다!

두 코드 모두 PyQt5로 GUI를 구현
FFT 결과를 실시간으로 시각화
두 번째 코드는 첫 번째 코드에 X축/Y축 범위 설정, 로그 스케일, 선 색상/두께 등의 커스터마이징 기능이 추가된 버전입니다.
'''